@@ -2,6 +2,7 @@
  * cifsiostat: Report I/O statistics for CIFS filesystems.
  * Copyright (C) 2010 Red Hat, Inc. All Rights Reserved
  * Written by Ivana Varekova <varekova@redhat.com>
+ * Maintained / multiple enhancements by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -33,8 +34,8 @@
 #include "rd_stats.h"
 #include "count.h"
 
+#include <locale.h>	/* For setlocale() */
 #ifdef USE_NLS
-#include <locale.h>
 #include <libintl.h>
 #define _(string) gettext(string)
 #else
@@ -47,7 +48,6 @@ char *sccsid(void) { return (SCCSID); }
 #endif
 
 #ifdef TEST
-void int_handler(int n) { return; }
 extern int __env;
 #endif
 
@@ -55,13 +55,15 @@ unsigned long long uptime_cs[2] = {0, 0};
 struct io_cifs *cifs_list = NULL;
 
 int cpu_nr = 0;		/* Nb of processors on the machine */
-int flags = 0;		/* Flag for common options and system state */
+uint64_t flags = 0;	/* Flag for common options and system state */
+uint64_t xflags = 0;	/* Extended flag for options used by multiple commands */
 int dplaces_nr = -1;	/* Number of decimal places */
 
 long interval = 0;
 char timestamp[TIMESTAMP_LEN];
 
-struct sigaction alrm_act;
+struct sigaction alrm_act, int_act;
+int sigint_caught = 0;
 
 /*
  ***************************************************************************
@@ -78,12 +80,12 @@ void usage(char *progname)
 
 #ifdef DEBUG
 	fprintf(stderr, _("Options are:\n"
-			  "[ --dec={ 0 | 1 | 2 } ] [ --human ] [ --pretty ]\n"
-			  "[ -h ] [ -k | -m ] [ -t ] [ -V ] [ --debuginfo ]\n"));
+			  "[ --dec={ 0 | 1 | 2 } ] [ --human ] [ --pretty ] [ -o JSON ]\n"
+			  "[ -h ] [ -k | -m ] [ -t ] [ -V ] [ -y ] [ --debuginfo ]\n"));
 #else
 	fprintf(stderr, _("Options are:\n"
-			  "[ --dec={ 0 | 1 | 2 } ] [ --human ] [ --pretty ]\n"
-			  "[ -h ] [ -k | -m ] [ -t ] [ -V ]\n"));
+			  "[ --dec={ 0 | 1 | 2 } ] [ --human ] [ --pretty ] [ -o JSON ]\n"
+			  "[ -h ] [ -k | -m ] [ -t ] [ -V ] [ -y ]\n"));
 #endif
 	exit(1);
 }
@@ -101,6 +103,19 @@ void alarm_handler(int sig)
 	alarm(interval);
 }
 
+/*
+ * **************************************************************************
+ * SIGINT signal handler.
+ *
+ * IN:
+ * @sig	Signal number.
+ **************************************************************************
+ */
+void int_handler(int sig)
+{
+	sigint_caught = 1;
+}
+
 /*
  ***************************************************************************
  * Set every cifs entry to nonexistent status.
@@ -193,9 +208,12 @@ struct io_cifs *add_list_cifs(struct io_cifs **clist, char *name)
  *
  * IN:
  * @curr	Index in array for current sample statistics.
+ *
+ * RETURNS:
+ * 1 if no CIFS filesystems found, and 0 otherwise.
  ***************************************************************************
  */
-void read_cifs_stat(int curr)
+int read_cifs_stat(int curr)
 {
 	FILE *fp;
 	char line[256];
@@ -209,7 +227,7 @@ void read_cifs_stat(int curr)
 	struct io_cifs *ci;
 
 	if ((fp = fopen(CIFSSTATS, "r")) == NULL)
-		return;
+		return 1;
 
 	sprintf(aux, "%%*d) %%%ds",
 		MAX_NAME_LEN < 200 ? MAX_NAME_LEN - 1 : 200);
@@ -285,6 +303,7 @@ void read_cifs_stat(int curr)
 	}
 
 	fclose(fp);
+	return 0;
 }
 
 /*
@@ -293,26 +312,42 @@ void read_cifs_stat(int curr)
  *
  * OUT:
  * @fctr	Conversion factor.
+ * @tab		Number of tabs to print (JSON format only).
  ***************************************************************************
  */
-void write_cifs_stat_header(int *fctr)
+void write_cifs_stat_header(int *fctr, int *tab)
 {
+	char *units, *spc;
+
+	if (DISPLAY_KILOBYTES(flags)) {
+		*fctr = 1024;
+		units = "kB";
+		spc = "";
+	}
+	else if (DISPLAY_MEGABYTES(flags)) {
+		*fctr = 1024 * 1024;
+		units = "MB";
+		spc = "";
+	}
+	else {
+		*fctr = 1;
+		units = "B";
+		spc = " ";
+	}
+
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
+		xprintf((*tab)++, "\"filesystem\": [");
+		return;
+	}
+
 	if (!DISPLAY_PRETTY(flags)) {
 		printf("Filesystem            ");
 	}
-	if (DISPLAY_KILOBYTES(flags)) {
-		printf("        rkB/s        wkB/s");
-		*fctr = 1024;
-	}
-	else if (DISPLAY_MEGABYTES(flags)) {
-		printf("        rMB/s        wMB/s");
-		*fctr = 1024 * 1024;
-	}
-	else {
-		printf("         rB/s         wB/s");
-		*fctr = 1;
-	}
-	printf("    rops/s    wops/s         fo/s         fc/s         fd/s");
+
+	printf("        %sr%s/s        %sw%s/s"
+	       "    rops/s    wops/s         fo/s         fc/s         fd/s",
+	       spc, units, spc, units);
+
 	if (DISPLAY_PRETTY(flags)) {
 		printf(" Filesystem");
 	}
@@ -320,8 +355,8 @@ void write_cifs_stat_header(int *fctr)
 }
 
 /*
- ***************************************************************************
- * Write CIFS stats read from /proc/fs/cifs/Stats.
+ * **************************************************************************
+ * Write CIFS stats read from /proc/fs/cifs/Stats in plain format.
  *
  * IN:
  * @curr	Index in array for current sample statistics.
@@ -332,9 +367,9 @@ void write_cifs_stat_header(int *fctr)
  * @ioj		Previous sample statistics.
  ***************************************************************************
  */
-void write_cifs_stat(int curr, unsigned long long itv, int fctr,
-		     struct io_cifs *clist, struct cifs_st *ioni,
-		     struct cifs_st *ionj)
+void write_plain_cifs_stat(int curr, unsigned long long itv, int fctr,
+			   struct io_cifs *clist, struct cifs_st *ioni,
+			   struct cifs_st *ionj)
 {
 	double rbytes, wbytes;
 
@@ -364,6 +399,91 @@ void write_cifs_stat(int curr, unsigned long long itv, int fctr,
 	printf("\n");
 }
 
+/*
+ * **************************************************************************
+ * Write CIFS stats read from /proc/fs/cifs/Stats in JSON format.
+ *
+ * IN:
+ * @tab		Number of tabs to print.
+ * @curr	Index in array for current sample statistics.
+ * @itv		Interval of time (in 1/100th of a second).
+ * @fctr	Conversion factor.
+ * @clist	Pointer on the linked list where the cifs is saved.
+ * @ioi		Current sample statistics.
+ * @ioj		Previous sample statistics.
+ ***************************************************************************
+ */
+void write_json_cifs_stat(int tab, int curr, unsigned long long itv, int fctr,
+			  struct io_cifs *clist, struct cifs_st *ioni,
+			  struct cifs_st *ionj)
+{
+	char line[256];
+
+	xprintf0(tab,
+		 "{\"fs_name\": \"%s\", ", escape_bs_char(clist->name));
+
+	if (DISPLAY_KILOBYTES(flags)) {
+		sprintf(line, "\"rkB/s\": %%.2f, \"wkB/s\": %%.2f, ");
+	}
+	else if (DISPLAY_MEGABYTES(flags)) {
+		sprintf(line, "\"rMB/s\": %%.2f, \"wMB/s\": %%.2f, ");
+	}
+	else {
+		sprintf(line, "\"rB/s\": %%.2f, \"wB/s\": %%.2f, ");
+	}
+	printf(line,
+	       S_VALUE(ionj->rd_bytes, ioni->rd_bytes, itv) / fctr,
+	       S_VALUE(ionj->wr_bytes, ioni->wr_bytes, itv) / fctr);
+
+	printf("\"rops/s\": %.2f, \"wops/s\": %.2f, "
+	       "\"fo/s\": %.2f, \"fc/s\": %.2f, \"fd/s\": %.2f}",
+	       S_VALUE(ionj->rd_ops, ioni->rd_ops, itv),
+	       S_VALUE(ionj->wr_ops, ioni->wr_ops, itv),
+	       S_VALUE(ionj->fopens, ioni->fopens, itv),
+	       S_VALUE(ionj->fcloses, ioni->fcloses, itv),
+	       S_VALUE(ionj->fdeletes, ioni->fdeletes, itv));
+}
+
+/*
+ ***************************************************************************
+ * Write CIFS stats read from /proc/fs/cifs/Stats in plain or JSON format.
+ *
+ * IN:
+ * @curr	Index in array for current sample statistics.
+ * @itv		Interval of time (in 1/100th of a second).
+ * @fctr	Conversion factor.
+ * @clist	Pointer on the linked list where the cifs is saved.
+ * @ioi		Current sample statistics.
+ * @ioj		Previous sample statistics.
+ * @tab		Number of tabs to print (JSON format only).
+ ***************************************************************************
+ */
+void write_cifs_stat(int curr, unsigned long long itv, int fctr,
+		     struct io_cifs *clist, struct cifs_st *ioni,
+		     struct cifs_st *ionj, int tab)
+{
+#ifdef DEBUG
+	if (DISPLAY_DEBUG(xflags)) {
+		/* Debug output */
+		fprintf(stderr, "name=%s itv=%llu fctr=%d ioni{ rd_bytes=%llu "
+		"wr_bytes=%llu rd_ops=%llu wr_ops=%llu fopens=%llu "
+		"fcloses=%llu fdeletes=%llu}\n",
+	  clist->name, itv, fctr,
+	  ioni->rd_bytes, ioni->wr_bytes,
+	  ioni->rd_ops,   ioni->wr_ops,
+	  ioni->fopens,   ioni->fcloses,
+	  ioni->fdeletes);
+	}
+#endif
+
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
+		write_json_cifs_stat(tab, curr, itv, fctr, clist, ioni, ionj);
+	}
+	else {
+		write_plain_cifs_stat(curr, itv, fctr, clist, ioni, ionj);
+	}
+}
+
 /*
  ***************************************************************************
  * Print everything now (stats and uptime).
@@ -375,7 +495,7 @@ void write_cifs_stat(int curr, unsigned long long itv, int fctr,
  */
 void write_stats(int curr, struct tm *rectime)
 {
-	int fctr = 1;
+	int fctr = 1, tab = 4, next = FALSE;
 	unsigned long long itv;
 	struct io_cifs *clist;
 	struct cifs_st *ioni, *ionj;
@@ -383,27 +503,20 @@ void write_stats(int curr, struct tm *rectime)
 	/* Test stdout */
 	TEST_STDOUT(STDOUT_FILENO);
 
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
+		xprintf(tab++, "{");
+	}
+
 	/* Print time stamp */
 	if (DISPLAY_TIMESTAMP(flags)) {
-		if (DISPLAY_ISO(flags)) {
-			strftime(timestamp, sizeof(timestamp), "%FT%T%z", rectime);
-		}
-		else {
-			strftime(timestamp, sizeof(timestamp), "%x %X", rectime);
-		}
-		printf("%s\n", timestamp);
-#ifdef DEBUG
-		if (DISPLAY_DEBUG(flags)) {
-			fprintf(stderr, "%s\n", timestamp);
-		}
-#endif
+		write_sample_timestamp(tab, rectime, xflags);
 	}
 
 	/* Interval of time, reduced to one processor */
 	itv = get_interval(uptime_cs[!curr], uptime_cs[curr]);
 
 	/* Display CIFS stats header */
-	write_cifs_stat_header(&fctr);
+	write_cifs_stat_header(&fctr, &tab);
 
 	for (clist = cifs_list; clist != NULL; clist = clist->next) {
 
@@ -414,22 +527,19 @@ void write_stats(int curr, struct tm *rectime)
 		ioni = clist->cifs_stats[curr];
 		ionj = clist->cifs_stats[!curr];
 
-#ifdef DEBUG
-		if (DISPLAY_DEBUG(flags)) {
-			/* Debug output */
-			fprintf(stderr, "name=%s itv=%llu fctr=%d ioni{ rd_bytes=%llu "
-					"wr_bytes=%llu rd_ops=%llu wr_ops=%llu fopens=%llu "
-					"fcloses=%llu fdeletes=%llu}\n",
-				clist->name, itv, fctr,
-				ioni->rd_bytes, ioni->wr_bytes,
-				ioni->rd_ops,   ioni->wr_ops,
-				ioni->fopens,   ioni->fcloses,
-				ioni->fdeletes);
+		if (DISPLAY_JSON_OUTPUT(xflags) && next) {
+			printf(",\n");
 		}
-#endif
-		write_cifs_stat(curr, itv, fctr, clist, ioni, ionj);
+		next = TRUE;
+
+		write_cifs_stat(curr, itv, fctr, clist, ioni, ionj, tab);
+	}
+
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
+		printf("\n");
+		xprintf(--tab, "]");
+		xprintf0(--tab, "}");
 	}
-	printf("\n");
 }
 
 /*
@@ -444,6 +554,12 @@ void write_stats(int curr, struct tm *rectime)
 void rw_io_stat_loop(long int count, struct tm *rectime)
 {
 	int curr = 1;
+	int skip = 0;
+
+	/* Should we skip first report? */
+	if (DISPLAY_OMIT_SINCE_BOOT(flags) && interval > 0) {
+		skip = 1;
+	}
 
 	/* Set a handler for SIGALRM */
 	memset(&alrm_act, 0, sizeof(alrm_act));
@@ -451,6 +567,11 @@ void rw_io_stat_loop(long int count, struct tm *rectime)
 	sigaction(SIGALRM, &alrm_act, NULL);
 	alarm(interval);
 
+	/* Set a handler for SIGINT */
+	memset(&int_act, 0, sizeof(int_act));
+	int_act.sa_handler = int_handler;
+	sigaction(SIGINT, &int_act, NULL);
+
 	do {
 		/* Every device is potentially nonexistent */
 		set_cifs_nonexistent(cifs_list);
@@ -459,24 +580,52 @@ void rw_io_stat_loop(long int count, struct tm *rectime)
 		read_uptime(&(uptime_cs[curr]));
 
 		/* Read CIFS stats */
-		read_cifs_stat(curr);
+		if (read_cifs_stat(curr)) {
+			/* No CIFS fs found */
+			if (!DISPLAY_JSON_OUTPUT(xflags)) {
+				fprintf(stderr, _("No CIFS filesystems with statistics found\n"));
+				exit(1);
+			}
+			/*
+			 * Don't exit now if displaying stats in JSON format so that
+			 * JSON file can be properly terminated.
+			 */
+			count = 0;
+		}
 
 		/* Get time */
 		get_xtime(rectime, 0, LOCAL_TIME);
 
-		/* Print results */
-		write_stats(curr, rectime);
+		/* Check whether we should skip first report */
+		if (!skip) {
+			/* Print results */
+			write_stats(curr, rectime);
 
-		if (count > 0) {
-			count--;
+			if (count > 0) {
+				count--;
+			}
 		}
 
 		if (count) {
 			curr ^= 1;
 			__pause();
+
+			if (sigint_caught) {
+				/* SIGINT signal caught => Terminate JSON output properly */
+				count = 0;
+			}
+			else if (DISPLAY_JSON_OUTPUT(xflags) && !skip) {	/* count != 0 */
+				printf(",");
+			}
 		}
+		skip = 0;
+		printf("\n");
 	}
 	while (count);
+
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
+		printf("\t\t\t]\n\t\t}\n\t]\n}}\n");
+	}
 }
 
 /*
@@ -506,7 +655,7 @@ int main(int argc, char **argv)
 
 #ifdef DEBUG
 		if (!strcmp(argv[opt], "--debuginfo")) {
-			flags |= I_D_DEBUG;
+			xflags |= X_D_DEBUG;
 			opt++;
 		} else
 #endif
@@ -523,6 +672,17 @@ int main(int argc, char **argv)
 		}
 #endif
 
+		else if (!strcmp(argv[opt], "-o")) {
+			/* Select output format */
+			if (argv[++opt] && !strcmp(argv[opt], K_JSON)) {
+				xflags |= X_D_JSON_OUTPUT;
+				opt++;
+			}
+			else {
+				usage(argv[0]);
+			}
+		}
+
 		else if (!strcmp(argv[opt], "--pretty")) {
 			/* Display an easy-to-read CIFS report */
 			flags |= I_D_PRETTY;
@@ -574,6 +734,11 @@ int main(int argc, char **argv)
 					flags |= I_D_TIMESTAMP;
 					break;
 
+				case 'y':
+					/* Don't display stats since system restart */
+					flags |= I_D_OMIT_SINCE_BOOT;
+					break;
+
 				case 'V':
 					{
 						char *cifsiostat_env[] = {ENV_COLORS,
@@ -629,14 +794,23 @@ int main(int argc, char **argv)
 	 */
 	setbuf(stdout, NULL);
 
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
+		/* Use a decimal point to make JSON code compliant with RFC7159 */
+		setlocale(LC_NUMERIC, "C");
+	}
+
 	/* Get system name, release number and hostname */
 	__uname(&header);
 	if (print_gal_header(&rectime, header.sysname, header.release,
 			     header.nodename, header.machine, cpu_nr,
-			     PLAIN_OUTPUT)) {
-		flags |= I_D_ISO;
+			     DISPLAY_JSON_OUTPUT(xflags))) {
+		xflags |= X_D_ISO;
 	}
-	printf("\n");
+
+	if (!DISPLAY_JSON_OUTPUT(xflags) &&
+		(!DISPLAY_OMIT_SINCE_BOOT(flags) || (interval == 0))) {
+		printf("\n");
+		}
 
 	/* Main loop */
 	rw_io_stat_loop(count, &rectime);
