@@ -1,6 +1,6 @@
 /*
  * sar, sadc, sadf, mpstat and iostat common routines.
- * (C) 1999-2023 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2025 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -442,22 +442,56 @@ int check_dir(char *dirname)
  * @val1	First value.
  * @val2	Second value.
  * @val3	Third value.
+ *
+ * RETURNS:
+ * Multiplication of the 3 values.
  ***************************************************************************
  */
-void check_overflow(unsigned int val1, unsigned int val2,
-		    unsigned int val3)
+size_t mul_check_overflow3(size_t val1, size_t val2, size_t val3)
 {
 	if ((val1 != 0) && (val2 != 0) && (val3 != 0) &&
-	    (((unsigned long long) UINT_MAX / (unsigned long long) val1 <
-	      (unsigned long long) val2) ||
-	     ((unsigned long long) UINT_MAX / ((unsigned long long) val1 * (unsigned long long) val2) <
-	      (unsigned long long) val3))) {
+	    ((UINT_MAX < val1) ||
+	     (UINT_MAX / val1 < val2) ||
+	     (UINT_MAX / (val1 * val2) < val3))) {
 #ifdef DEBUG
-		fprintf(stderr, "%s: Overflow detected (%u,%u,%u). Aborting...\n",
+		fprintf(stderr, "%s: Overflow detected (%zu,%zu,%zu). Aborting...\n",
 			__FUNCTION__, val1, val2, val3);
 #endif
 		exit(4);
 	}
+
+	return (val1 * val2 * val3);
+}
+
+/*
+ * **************************************************************************
+ * Check if the multiplication of the 4 values may be greater than UINT_MAX.
+ *
+ * IN:
+ * @val1	First value.
+ * @val2	Second value.
+ * @val3	Third value.
+ * @val4	Fourth value.
+ *
+ * RETURNS:
+ * Multiplication of the 4 values.
+ ***************************************************************************
+ */
+size_t mul_check_overflow4(size_t val1, size_t val2, size_t val3, size_t val4)
+{
+	if ((val1 != 0) && (val2 != 0) && (val3 != 0) && (val4 != 0) &&
+	    ((UINT_MAX < val1) ||
+	     (UINT_MAX / val1 < val2) ||
+	     (UINT_MAX / (val1 * val2) < val3) ||
+	     (UINT_MAX / (val1 * val2 * val3) < val4))) {
+#ifdef DEBUG
+		fprintf(stderr, "%s: Overflow detected (%zu,%zu,%zu,%zu). Aborting...\n",
+			__FUNCTION__, val1, val2, val3, val4);
+#endif
+		exit(4);
+	}
+
+	return (val1 * val2 * val3 * val4);
 }
 
 #ifndef SOURCE_SADC
@@ -746,6 +780,41 @@ char *device_name(char *name)
 	return out;
 }
 
+/*
+ ***************************************************************************
+ * Escape a '\' character in a JSON string (replace '\' with "\\").
+ *
+ * IN:
+ * @str		String which may contain '\' characters.
+ *
+ * RETURNS:
+ * String where '\' characters have been escaped.
+ ***************************************************************************
+ */
+char *escape_bs_char(char *str)
+{
+	static char buffer[MAX_NAME_LEN];
+	int i = 0, j = 0;
+
+	while (str[i] != '\0' && j < MAX_NAME_LEN - 1) {
+		if (str[i] == '\\') {
+			if (j < MAX_NAME_LEN - 2) {
+				buffer[j++] = '\\';
+				buffer[j++] = '\\';
+			} else {
+				break;
+			}
+		} else {
+			buffer[j++] = str[i];
+		}
+		i++;
+	}
+
+	buffer[j] = '\0';
+
+	return buffer;
+}
+
 /*
  ***************************************************************************
  * Workaround for CPU counters read from /proc/stat: Dyn-tick kernels
@@ -1755,7 +1824,7 @@ int parse_values(char *strargv, unsigned char bitmap[], int max_val, const char
 				return 1;
 
 			for (i = val_low; i <= val; i++) {
-				bitmap[(i + 1) >> 3] |= 1 << ((i + 1) & 0x07);
+				SET_CPU_BITMAP(bitmap, i + 1);
 			}
 		}
 	}
@@ -1763,4 +1832,38 @@ int parse_values(char *strargv, unsigned char bitmap[], int max_val, const char
 	return 0;
 }
 
+/*
+ * **************************************************************************
+ * Write current sample's timestamp, either in plain or JSON format.
+ *
+ * IN:
+ * @tab		Number of tabs to print.
+ * @rectime	Current date and time.
+ * @xflags	Flag for common options and system state.
+ ***************************************************************************
+ */
+void write_sample_timestamp(int tab, struct tm *rectime, uint64_t xflags)
+{
+	char timestamp[TIMESTAMP_LEN];
+
+	if (DISPLAY_ISO(xflags)) {
+		strftime(timestamp, sizeof(timestamp), "%FT%T%z", rectime);
+	}
+	else {
+		strftime(timestamp, sizeof(timestamp), "%x %X", rectime);
+	}
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
+		xprintf(tab, "\"timestamp\": \"%s\",", timestamp);
+	}
+	else {
+		printf("%s\n", timestamp);
+	}
+
+#ifdef DEBUG
+	if (DISPLAY_DEBUG(xflags)) {
+		fprintf(stderr, "%s\n", timestamp);
+	}
+#endif
+}
+
 #endif /* SOURCE_SADC undefined */
