@@ -1,6 +1,6 @@
 /*
  * sysstat: System performance tools for Linux
- * (C) 1999-2023 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2025 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _COMMON_H
@@ -9,10 +9,12 @@
 /* Maximum length of sensors device name */
 #define MAX_SENSORS_DEV_LEN	20
 
+#include <stdio.h>
 #include <time.h>
 #include <sched.h>	/* For __CPU_SETSIZE */
 #include <limits.h>
 #include <stdlib.h>
+#include <stdint.h>
 #include "systest.h"
 
 #ifdef HAVE_SYS_SYSMACROS_H
@@ -71,6 +73,15 @@ enum {
 #define K_UTC		"UTC"
 #define K_JSON		"JSON"
 
+/* Flags used with multiple commands (iostat, cifsiostat...) */
+#define X_D_DEBUG		0x01
+#define X_D_ISO			0x02
+#define X_D_JSON_OUTPUT		0x04
+
+#define DISPLAY_DEBUG(m)	(((m) & X_D_DEBUG)       == X_D_DEBUG)
+#define DISPLAY_ISO(m)		(((m) & X_D_ISO)         == X_D_ISO)
+#define DISPLAY_JSON_OUTPUT(m)	(((m) & X_D_JSON_OUTPUT) == X_D_JSON_OUTPUT)
+
 /* Files */
 #define __DISKSTATS		"diskstats"
 #define __BLOCK			"block"
@@ -138,9 +149,30 @@ enum {
 #define BITMAP_SIZE(m)	((((m) + 1) >> 3) + 1)
 
 /* Allocate and init structure */
+#ifdef DEBUG
 #define SREALLOC(S, TYPE, SIZE)	do {								   \
-					TYPE *_p_;						   \
-					if ((SIZE) != 0) {					   \
+					size_t _sz_ = (SIZE);					   \
+					if ((_sz_) == 0) {					   \
+						 /* SIZE may be zero when an overflow happens */   \
+						 /* when two non-zero values are multiplied   */   \
+						 /* together.				      */   \
+						 fprintf(stderr, "%s: SREALLOC: SIZE is zero!\n", __FUNCTION__); \
+						 exit(4); 					   \
+					}							   \
+					__SREALLOC(S, TYPE, _sz_);				   \
+				} while (0)
+#else
+#define SREALLOC(S, TYPE, SIZE)	do {								   \
+					size_t _sz_ = (SIZE);					   \
+					if ((_sz_) == 0) {					   \
+						exit(4); 					   \
+					}							   \
+					__SREALLOC(S, TYPE, _sz_);				   \
+				} while (0)
+#endif
+
+#define __SREALLOC(S, TYPE, SIZE)	do {							   \
+						TYPE *_p_;					   \
 						if ((_p_ = (TYPE *) realloc(S, (SIZE))) == NULL) { \
 							perror("realloc");			   \
 							if (S) {				   \
@@ -153,8 +185,18 @@ enum {
 							memset(_p_, 0, (SIZE));			   \
 						}						   \
 						S = _p_;					   \
-					}							   \
-				} while (0)
+					} while (0)
+
+/* Set CPU in given bitmap   */
+#define SET_CPU_BITMAP(bitmap, cpu)	bitmap[(cpu) >> 3] |= 1 << ((cpu) & 0x07)
+/* Check if CPU is set in bitmap */
+#define IS_CPU_SET(bitmap, cpu)		(bitmap[(cpu) >> 3] & (1 << ((cpu) & 0x07)))
+/* Mark CPU as offline in dedicated CPU bitmap */
+#define MARK_CPU_OFFLINE(bitmap, cpu) 	SET_CPU_BITMAP(bitmap, cpu)
+/* Check if given CPU is offline */
+#define IS_CPU_OFFLINE(bitmap, cpu)	IS_CPU_SET(bitmap, cpu)
+/* Check if given CPU has been selected */
+#define IS_CPU_SELECTED(bitmap, cpu)	IS_CPU_SET(bitmap, cpu)
 
 /*
  * Macros used to display statistics values.
@@ -254,7 +296,7 @@ struct ext_disk_stats {
  ***************************************************************************
  */
 void print_version
-	(void);
+	(char *[], int);
 void get_HZ
 	(void);
 void get_kb_shift
@@ -275,8 +317,10 @@ int get_wwnid_from_pretty
 	(char *, unsigned long long *, unsigned int *);
 int check_dir
 	(char *);
-void check_overflow
-	(unsigned int, unsigned int, unsigned int);
+size_t mul_check_overflow3
+	(size_t, size_t, size_t);
+size_t mul_check_overflow4
+	(size_t, size_t, size_t, size_t);
 
 #ifndef SOURCE_SADC
 int count_bits
@@ -299,6 +343,8 @@ void cprintf_tr
 	(int, char *, char *);
 char *device_name
 	(char *);
+char *escape_bs_char
+	(char *);
 char *get_device_name
 	(unsigned int, unsigned int, unsigned long long [],
 	 unsigned int, unsigned int, unsigned int, unsigned int, char *);
@@ -332,6 +378,8 @@ int set_report_date
 	(struct tm *, char[], int);
 char *strtolower
 	(char *);
+void write_sample_timestamp
+	(int, struct tm *, uint64_t);
 void xprintf
 	(int, const char *, ...);
 void xprintf0
