@@ -1,6 +1,6 @@
 /*
  * json_stats.c: Functions used by sadf to display statistics in JSON format.
- * (C) 1999-2023 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2025 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -24,7 +24,6 @@
 #include <stdarg.h>
 
 #include "sa.h"
-#include "ioconf.h"
 
 #ifdef USE_NLS
 #include <locale.h>
@@ -162,8 +161,8 @@ __print_funct_t json_print_cpu_stats(struct activity *a, int curr, int tab,
 	for (i = 0; (i < a->nr_ini) && (i < a->bitmap->b_size + 1); i++) {
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))) ||
-		    offline_cpu_bitmap[i >> 3] & (1 << (i & 0x07)))
+		if (!IS_CPU_SELECTED(a->bitmap->b_array, i) ||
+		    IS_CPU_OFFLINE(offline_cpu_bitmap, i))
 			/* Don't display CPU */
 			continue;
 
@@ -368,7 +367,7 @@ __print_funct_t json_print_irq_stats(struct activity *a, int curr, int tab,
 										  + i * a->msize);
 
 			/* Should current CPU (including CPU "all") be displayed? */
-			if (masked_cpu_bitmap[c >> 3] & (1 << (c & 0x07)))
+			if (IS_CPU_SET(masked_cpu_bitmap, c))
 				/* No */
 				continue;
 
@@ -1465,7 +1464,7 @@ __print_funct_t json_print_net_etcp_stats(struct activity *a, int curr, int tab,
 	xprintf0(tab, "\"net-etcp\": {"
 		 "\"atmptf\": %.2f, "
 		 "\"estres\": %.2f, "
-		 "\"retrans\": %.2f, "
+		 "\"retrseg\": %.2f, "
 		 "\"isegerr\": %.2f, "
 		 "\"orsts\": %.2f}",
 		 S_VALUE(snetp->AttemptFails, snetc->AttemptFails,  itv),
@@ -1865,7 +1864,7 @@ __print_funct_t json_print_pwr_cpufreq_stats(struct activity *a, int curr, int t
 		spc = (struct stats_pwr_cpufreq *) ((char *) a->buf[curr] + i * a->msize);
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))))
+		if (!IS_CPU_SELECTED(a->bitmap->b_array, i))
 			/* No */
 			continue;
 
@@ -2129,7 +2128,7 @@ __print_funct_t json_print_pwr_wghfreq_stats(struct activity *a, int curr, int t
 		spp = (struct stats_pwr_wghfreq *) ((char *) a->buf[!curr] + i * a->msize * a->nr2);
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))))
+		if (!IS_CPU_SELECTED(a->bitmap->b_array, i))
 			/* No */
 			continue;
 
@@ -2435,8 +2434,8 @@ __print_funct_t json_print_softnet_stats(struct activity *a, int curr, int tab,
 		 * used by sadc to create a file, and the version of sysstat
 		 * used by sar to read it...
 		 */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))) ||
-		    offline_cpu_bitmap[i >> 3] & (1 << (i & 0x07)))
+		if (!IS_CPU_SELECTED(a->bitmap->b_array, i) ||
+		    IS_CPU_OFFLINE(offline_cpu_bitmap, i))
 			/* No */
 			continue;
 
