@@ -1,6 +1,6 @@
 /*
  * mpstat: per-processor statistics
- * (C) 2000-2023 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 2000-2025 by Sebastien GODARD (sysstat <at> orange.fr)
  * Copyright (C) 2022 Oracle and/or its affiliates.
  *
  ***************************************************************************
@@ -25,7 +25,6 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <signal.h>
-#include <errno.h>
 #include <dirent.h>
 #include <ctype.h>
 #include <sys/utsname.h>
@@ -94,9 +93,10 @@ struct cpu_topology *st_cpu_topology = NULL;
 struct tm mp_tstamp[3];
 
 /* Activity flag */
-unsigned int actflags = 0;
+uint64_t actflags = 0;
 
-unsigned int flags = 0;
+uint64_t flags = 0;
+uint64_t xflags = 0;	/* Extended flag for options used by multiple commands */
 
 /* Interval and count parameters */
 long interval = -1, count = 0;
@@ -571,7 +571,7 @@ unsigned long long get_global_cpu_mpstats(int prev, int curr,
 			 * Mark CPU as offline to not display it
 			 * (and thus it will not be confused with a tickless CPU).
 			 */
-			offline_cpu_bitmap[i >> 3] |= 1 << (i & 0x07);
+			MARK_CPU_OFFLINE(offline_cpu_bitmap, i);
 		}
 
 		if ((tot_jiffies_p == 0) && (interval != 0)) {
@@ -585,7 +585,7 @@ unsigned long long get_global_cpu_mpstats(int prev, int curr,
 			 * NB: Test for interval != 0 to make sure we don't want stats
 			 * since boot time.
 			 */
-			offline_cpu_bitmap[i >> 3] |= 1 << (i & 0x07);
+			MARK_CPU_OFFLINE(offline_cpu_bitmap, i);
 			continue;
 		}
 
@@ -649,7 +649,7 @@ void write_plain_cpu_stats(int dis, unsigned long long deltot_jiffies, int prev,
 
 		/* Check if we want stats about this proc */
 		if (!(*(cpu_bitmap + (i >> 3)) & (1 << (i & 0x07))) ||
-		    offline_cpu_bitmap[i >> 3] & (1 << (i & 0x07)))
+		    IS_CPU_OFFLINE(offline_cpu_bitmap, i))
 			continue;
 
 		scc = st_cpu[curr] + i;
@@ -757,7 +757,7 @@ void write_json_cpu_stats(int tab, unsigned long long deltot_jiffies, int prev,
 
 		/* Check if we want stats about this proc */
 		if (!(*(cpu_bitmap + (i >> 3)) & (1 << (i & 0x07))) ||
-		    offline_cpu_bitmap[i >> 3] & (1 << (i & 0x07)))
+		    IS_CPU_OFFLINE(offline_cpu_bitmap, i))
 			continue;
 
 		scc = st_cpu[curr] + i;
@@ -877,7 +877,7 @@ void write_cpu_stats(int dis, unsigned long long deltot_jiffies, int prev, int c
 		deltot_jiffies = 1;
 	}
 
-	if (DISPLAY_JSON_OUTPUT(flags)) {
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
 		if (*next) {
 			printf(",\n");
 		}
@@ -1142,7 +1142,7 @@ void write_node_stats(int dis, unsigned long long deltot_jiffies, int prev, int
 		deltot_jiffies = 1;
 	}
 
-	if (DISPLAY_JSON_OUTPUT(flags)) {
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
 		if (*next) {
 			printf(",\n");
 		}
@@ -1208,7 +1208,7 @@ void write_plain_isumcpu_stats(int dis, unsigned long long itv, int prev, int cu
 
 		/* Check if we want stats about this CPU */
 		if (!(*(cpu_bitmap + (cpu >> 3)) & (1 << (cpu & 0x07))) ||
-		    offline_cpu_bitmap[cpu >> 3] & (1 << (cpu & 0x07)))
+		    IS_CPU_OFFLINE(offline_cpu_bitmap, cpu))
 			continue;
 
 		printf("%-11s", curr_string);
@@ -1274,7 +1274,7 @@ void write_json_isumcpu_stats(int tab, unsigned long long itv, int prev, int cur
 
 		/* Check if we want stats about this CPU */
 		if (!(*(cpu_bitmap + (cpu >> 3)) & (1 << (cpu & 0x07))) ||
-		    offline_cpu_bitmap[cpu >> 3] & (1 << (cpu & 0x07)))
+		    IS_CPU_OFFLINE(offline_cpu_bitmap, cpu))
 			continue;
 
 		if (next) {
@@ -1329,7 +1329,7 @@ void write_isumcpu_stats(int dis, unsigned long long itv, int prev, int curr,
 			 char *prev_string, char *curr_string, int tab, int *next,
 			 unsigned char offline_cpu_bitmap[])
 {
-	if (DISPLAY_JSON_OUTPUT(flags)) {
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
 		if (*next) {
 			printf(",\n");
 		}
@@ -1433,7 +1433,7 @@ void write_plain_irqcpu_stats(struct stats_irqcpu *st_ic[], int ic_nr, int dis,
 		 * else we display every CPU (unless it's offline).
 		 */
 		if ((!(*(cpu_bitmap + (cpu >> 3)) & (1 << (cpu & 0x07))) && USE_OPTION_P(flags)) ||
-		    offline_cpu_bitmap[cpu >> 3] & (1 << (cpu & 0x07)))
+		    IS_CPU_OFFLINE(offline_cpu_bitmap, cpu))
 			continue;
 
 		printf("%-11s", curr_string);
@@ -1528,7 +1528,7 @@ void write_json_irqcpu_stats(int tab, struct stats_irqcpu *st_ic[], int ic_nr,
 		 * else we display every CPU (unless it's offline).
 		 */
 		if ((!(*(cpu_bitmap + (cpu >> 3)) & (1 << (cpu & 0x07))) && USE_OPTION_P(flags)) ||
-		    offline_cpu_bitmap[cpu >> 3] & (1 << (cpu & 0x07)))
+		    IS_CPU_OFFLINE(offline_cpu_bitmap, cpu))
 			continue;
 
 		if (nextcpu) {
@@ -1629,7 +1629,7 @@ void write_irqcpu_stats(struct stats_irqcpu *st_ic[], int ic_nr, int dis,
 			char *prev_string, char *curr_string, int tab,
 			int *next, int type, unsigned char offline_cpu_bitmap[])
 {
-	if (DISPLAY_JSON_OUTPUT(flags)) {
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
 		if (*next) {
 			printf(",\n");
 		}
@@ -1677,7 +1677,7 @@ void write_stats_core(int prev, int curr, int dis,
 	 */
 	deltot_jiffies = get_global_cpu_mpstats(prev, curr, offline_cpu_bitmap);
 
-	if (DISPLAY_JSON_OUTPUT(flags)) {
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
 		xprintf(tab++, "{");
 		xprintf(tab, "\"timestamp\": \"%s\",", curr_string);
 	}
@@ -1716,7 +1716,7 @@ void write_stats_core(int prev, int curr, int dis,
 				   offline_cpu_bitmap);
 	}
 
-	if (DISPLAY_JSON_OUTPUT(flags)) {
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
 		printf("\n");
 		xprintf0(--tab, "}");
 	}
@@ -2006,7 +2006,7 @@ void rw_mpstat_loop(int dis_hdr, int rows)
 			memset(st_softirqcpu[1], 0, STATS_IRQCPU_SIZE * (cpu_nr + 1) * softirqcpu_nr);
 		}
 		write_stats(0, DISP_HDR);
-		if (DISPLAY_JSON_OUTPUT(flags)) {
+		if (DISPLAY_JSON_OUTPUT(xflags)) {
 			printf("\n\t\t\t]\n\t\t}\n\t]\n}}\n");
 		}
 		exit(0);
@@ -2039,7 +2039,7 @@ void rw_mpstat_loop(int dis_hdr, int rows)
 
 	if (sigint_caught)
 		/* SIGINT signal caught during first interval: Exit immediately */
-		return;
+		goto terminate;
 
 	do {
 		/*
@@ -2101,7 +2101,7 @@ void rw_mpstat_loop(int dis_hdr, int rows)
 				count = 0;
 			}
 			else {
-				if (DISPLAY_JSON_OUTPUT(flags)) {
+				if (DISPLAY_JSON_OUTPUT(xflags)) {
 					printf(",\n");
 				}
 				curr ^= 1;
@@ -2134,12 +2134,14 @@ void rw_mpstat_loop(int dis_hdr, int rows)
 	while (count);
 
 	/* Write stats average */
-	if (DISPLAY_JSON_OUTPUT(flags)) {
-		printf("\n\t\t\t]\n\t\t}\n\t]\n}}\n");
-	}
-	else {
+	if (!DISPLAY_JSON_OUTPUT(xflags)) {
 		write_stats_avg(curr, dis_hdr);
 	}
+
+terminate:
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
+		printf("\n\t\t\t]\n\t\t}\n\t]\n}}\n");
+	}
 }
 
 /*
@@ -2231,7 +2233,7 @@ int main(int argc, char **argv)
 		else if (!strcmp(argv[opt], "-o")) {
 			/* Select output format */
 			if (argv[++opt] && !strcmp(argv[opt], K_JSON)) {
-				flags |= F_JSON_OUTPUT;
+				xflags |= X_D_JSON_OUTPUT;
 			}
 			else {
 				usage(argv[0]);
@@ -2361,7 +2363,7 @@ int main(int argc, char **argv)
 		actflags |= M_D_CPU;
 	}
 
-	if (count_bits(&actflags, sizeof(unsigned int)) > 1) {
+	if (count_bits(&actflags, sizeof(actflags)) > 1) {
 		dis_hdr = 9;
 	}
 
@@ -2400,7 +2402,7 @@ int main(int argc, char **argv)
 		interval = 0;
 	}
 
-	if (DISPLAY_JSON_OUTPUT(flags)) {
+	if (DISPLAY_JSON_OUTPUT(xflags)) {
 		/* Use a decimal point to make JSON code compliant with RFC7159 */
 		setlocale(LC_NUMERIC, "C");
 	}
@@ -2419,7 +2421,7 @@ int main(int argc, char **argv)
 	__uname(&header);
 	print_gal_header(&(mp_tstamp[0]), header.sysname, header.release,
 			 header.nodename, header.machine, get_cpu_nr(~0, FALSE),
-			 DISPLAY_JSON_OUTPUT(flags));
+			 DISPLAY_JSON_OUTPUT(xflags));
 
 	/* Main loop */
 	rw_mpstat_loop(dis_hdr, rows);
