@@ -1,6 +1,6 @@
 /*
  * mpstat: per-processor statistics
- * (C) 2000-2024 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 2000-2025 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _MPSTAT_H
@@ -47,8 +47,7 @@
 #define F_OPTION_P	0x01
 /* Indicate that option -A has been used */
 #define F_OPTION_A	0x02
-/* JSON output */
-#define F_JSON_OUTPUT	0x04
+/* Unused		0x04 */
 /* Indicate that option -N has been used */
 #define F_OPTION_N	0x08
 /* Display topology */
@@ -58,7 +57,6 @@
 
 #define USE_OPTION_P(m)		(((m) & F_OPTION_P) == F_OPTION_P)
 #define USE_OPTION_A(m)		(((m) & F_OPTION_A) == F_OPTION_A)
-#define DISPLAY_JSON_OUTPUT(m)	(((m) & F_JSON_OUTPUT) == F_JSON_OUTPUT)
 #define USE_OPTION_N(m)		(((m) & F_OPTION_N) == F_OPTION_N)
 #define DISPLAY_TOPOLOGY(m)	(((m) & F_TOPOLOGY) == F_TOPOLOGY)
 #define USE_OPTION_H(m)		(((m) & F_OPTION_H) == F_OPTION_H)
