@@ -1,6 +1,6 @@
 /*
  * pcp_def_metrics.c: Functions used by sadf to define PCP metrics
- * (C) 2019-2023 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 2019-2025 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -19,7 +19,6 @@
  ***************************************************************************
  */
 
-#include "common.h"
 #include "sa.h"
 
 #ifdef HAVE_PCP
@@ -269,7 +268,7 @@ void pcp_def_pwr_cpufreq_metrics(pmInDom indom)
 #endif /* HAVE_PCP */
 
 /*
- ***********************************************global****************************
+ ***************************************************************************
  * Define PCP metrics for CPU related statistics.
  *
  * IN:
@@ -288,9 +287,9 @@ void pcp_def_cpu_metrics(struct activity *a)
 		/*
 		 * Should current CPU (including CPU "all") be displayed?
 		 * NB: Offline not tested (they may be turned off and on within
-		 * the same file.
+		 * the same file).
 		 */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))))
+		if (!IS_CPU_SELECTED(a->bitmap->b_array, i))
 			/* CPU not selected */
 			continue;
 
