@@ -1,6 +1,6 @@
 /*
  * pcp_stats.c: Functions used by sadf to create PCP archive files.
- * (C) 2019-2023 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 2019-2025 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -78,8 +78,8 @@ __print_funct_t pcp_print_cpu_stats(struct activity *a, int curr)
 	for (i = 0; (i < a->nr_ini) && (i < a->bitmap->b_size + 1); i++) {
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))) ||
-		    offline_cpu_bitmap[i >> 3] & (1 << (i & 0x07)))
+		if (!IS_CPU_SELECTED(a->bitmap->b_array, i) ||
+		    IS_CPU_OFFLINE(offline_cpu_bitmap, i))
 			/* Don't display CPU */
 			continue;
 
@@ -231,7 +231,7 @@ __print_funct_t pcp_print_irq_stats(struct activity *a, int curr)
 										  + i * a->msize);
 
 			/* Should current CPU (including CPU "all") be processed? */
-			if (masked_cpu_bitmap[c >> 3] & (1 << (c & 0x07)))
+			if (IS_CPU_SET(masked_cpu_bitmap, c))
 				/* No */
 				continue;
 
@@ -1501,7 +1501,7 @@ __print_funct_t pcp_print_pwr_cpufreq_stats(struct activity *a, int curr)
 		spc = (struct stats_pwr_cpufreq *) ((char *) a->buf[curr] + i * a->msize);
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))))
+		if (!IS_CPU_SELECTED(a->bitmap->b_array, i))
 			/* No */
 			continue;
 
@@ -1832,8 +1832,8 @@ __print_funct_t pcp_print_softnet_stats(struct activity *a, int curr)
 	for (i = 0; (i < a->nr_ini) && (i < a->bitmap->b_size + 1); i++) {
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (!(a->bitmap->b_array[i >> 3] & (1 << (i & 0x07))) ||
-		    offline_cpu_bitmap[i >> 3] & (1 << (i & 0x07)))
+		if (!IS_CPU_SELECTED(a->bitmap->b_array, i) ||
+		    IS_CPU_OFFLINE(offline_cpu_bitmap, i))
 			/* No */
 			continue;
 
