@@ -21,6 +21,7 @@
  */
 
 #include <stdio.h>
+#include <float.h>
 
 #include "sa.h"
 
@@ -147,11 +148,16 @@ void print_irq_xstats(int ismax, struct activity *a, int curr, int irq, char *na
 	for (cpu = 0; (cpu < a->nr[curr]) && (cpu < a->bitmap->b_size + 1); cpu++) {
 
 		/* Should current CPU (including CPU "all") be displayed? */
-		if (masked_cpu_bitmap[cpu >> 3] & (1 << (cpu & 0x07)))
+		if (IS_CPU_SET(masked_cpu_bitmap, cpu))
 			/* No */
 			continue;
 
-		cprintf_f(NO_UNIT, FALSE, 1, 9, 2, *(spextr + (cpu * a->nr2 + irq) * a->xnr));
+		/* Print min/max values if available */
+		if ((*(spextr + (cpu * a->nr2 + irq) * a->xnr) != -DBL_MAX) &&
+		    (*(spextr + (cpu * a->nr2 + irq) * a->xnr) !=  DBL_MAX)) {
+			cprintf_f(NO_UNIT, FALSE, 1, 9, 2,
+				  *(spextr + (cpu * a->nr2 + irq) * a->xnr));
+		}
 	}
 
 	if (DISPLAY_PRETTY(flags)) {
