@@ -1,6 +1,6 @@
 /*
  * rd_sensors.c: Read sensors statistics
- * (C) 1999-2023 by Sebastien GODARD (sysstat <at> orange.fr)
+ * (C) 1999-2025 by Sebastien GODARD (sysstat <at> orange.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -19,10 +19,8 @@
  ***************************************************************************
  */
 
-#include <stdio.h>
 #include <string.h>
 
-#include "common.h"
 #include "rd_stats.h"
 #include "rd_sensors.h"
 
