@@ -1,15 +1,11 @@
 /*
  * sar/sadc: Report system activity
- * (C) 1999-2023 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2025 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _SA_H
 #define _SA_H
 
-#include <stdio.h>
-#include <stdint.h>
-
-#include "common.h"
 #include "rd_stats.h"
 #include "rd_sensors.h"
 
@@ -416,6 +412,7 @@ struct svg_parm {
 	int graph_no;				/* Total number of views already displayed */
 	int restart;				/* TRUE if we have just met a RESTART record */
 	int nr_act_dispd;			/* Number of activities that will be displayed */
+	int mock;				/* TRUE if no data shall be actually printed */
 	char hour;				/* Hour, minute and second (expressed in the */
 	char minute;				/* locale of the datafile creator) for first */
 	char second;				/* sample */
@@ -1359,9 +1356,15 @@ enum svg_graph_type {
 	SVG_BAR_GRAPH	= 2
 };
 
+#define MOCK_MODE	TRUE
+#define REAL_MODE	FALSE
+
 /* Maximum number of horizontal lines for the background grid */
 #define MAX_HLINES_NR	10
 
+/* Minimum SVG canvas height (in pixels) */
+#define MIN_CANVAS_HEIGHT	100
+
 /* Color palette constants */
 #define SVG_COLORS_IDX_MASK	0x0f
 #define SVG_COL_PALETTE_SIZE	24
@@ -1617,6 +1620,8 @@ void init_custom_color_palette
 	(void);
 void init_extrema_values
 	(struct activity *, int);
+void init_minmax_buf
+	(struct activity *, size_t, size_t);
 int next_slice
 	(unsigned long long, unsigned long long, int, long);
 void parse_sa_devices
